import os

from django.conf import settings
from django.contrib import admin
from django.http import HttpResponseRedirect
from django.template.loader import render_to_string
from django.template.response import TemplateResponse
from django.urls import path, reverse

from .forms import UploadFileForm
from .models import Audio, Container, Field, File, Info, Meta, Stream, Subtitle, Video


class MetaAdmin(admin.ModelAdmin):
    list_display = ("id", "version", "root")

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class InfoInlineAdmin(admin.StackedInline):
    model = Info
    extra = 0
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "duration",
                    "seekable",
                    "live",
                    "audio_streams",
                    "video_streams",
                    "subtitle_streams",
                ),
            },
        ),
        (
            "Topology",
            {
                "fields": ("streams",),
            },
        ),
    )
    readonly_fields = ("streams",)

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def streams(self, obj):
        children = Stream.objects.filter(info=obj, parent=None)
        content = render_to_string(
            "admin/flumes_django/info/streams.html", {"children": children}
        )
        return content

    streams.allow_tags = True

    class Media:
        css = {"all": ("admin/flumes_django/common/topology.css",)}


class FieldInlineAdmin(admin.TabularInline):
    model = Field
    extra = 0

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class StreamAdmin(admin.ModelAdmin):
    inlines = [
        FieldInlineAdmin,
    ]

    def get_list_display(self, request):
        return (
            "id",
            "type",
            "media_type",
        )

    def get_readonly_fields(self, request, obj=None):
        return ("children",)

    def get_fieldsets(self, request, obj=None):
        # Same fields as list without it
        fields = list(self.get_list_display(request))
        fields.remove("id")
        fields = tuple(fields)
        return (
            (
                None,
                {"fields": fields},
            ),
            (
                "Topology",
                {
                    "fields": ("children",),
                },
            ),
        )

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def children(self, obj):
        content = render_to_string(
            "admin/flumes_django/stream/children.html", {"root": obj}
        )
        return content

    children.allow_tags = True

    class Media:
        css = {"all": ("admin/flumes_django/common/topology.css",)}


class ContainerAdmin(StreamAdmin):
    pass


class VideoAdmin(StreamAdmin):
    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super().get_readonly_fields(request, obj)
        readonly_fields += ("framerate", "par")
        return readonly_fields

    def get_list_display(self, request):
        list_display = super().get_list_display(request)
        list_display += (
            "bitrate",
            "max_bitrate",
            "depth",
            "framerate",
            "width",
            "height",
            "par",
            "is_image",
            "is_interlaced",
        )
        return list_display

    def par(self, obj):
        return "{}/{}".format(obj.par_num, obj.par_denom)

    def framerate(self, obj):
        return "{}/{}".format(obj.framerate_num, obj.framerate_denom)


class AudioAdmin(StreamAdmin):
    def get_list_display(self, request):
        list_display = super().get_list_display(request)
        list_display += (
            "bitrate",
            "max_bitrate",
            "depth",
            "language",
            "channel_mask",
            "channels",
            "sample_rate",
        )
        return list_display


class SubtitleAdmin(StreamAdmin):
    def get_list_display(self, request):
        list_display = super().get_list_display(request)
        list_display += ("language",)
        return list_display


class FileAdmin(admin.ModelAdmin):
    list_display = ("id", "name")
    inlines = [
        InfoInlineAdmin,
    ]

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        base_urls = super().get_urls()
        urls = [
            path(
                "upload/",
                self.admin_site.admin_view(self.upload),
                name="flumes_django_file_upload",
            ),
        ]
        return urls + base_urls

    def upload(self, request):
        if request.method == "POST":
            form = UploadFileForm(request.POST, request.FILES)
            if form.is_valid():
                meta = Meta.objects.all()[0]
                root = getattr(settings, "FLUMES_DJANGO_ROOT", meta.root)
                # Upload the file to the Flume storage
                f = request.FILES["file"]
                name = os.path.join(root, f.name)
                with open(name, "wb+") as destination:
                    for chunk in f.chunks():
                        destination.write(chunk)
                return HttpResponseRedirect(
                    reverse("admin:flumes_django_file_changelist")
                )
        else:
            form = UploadFileForm()
            opts = self.model._meta
            context = {
                **self.admin_site.each_context(request),
                "opts": opts,
                "has_view_permission": self.has_view_permission(request),
                "form": form,
            }
            return TemplateResponse(
                request, "admin/flumes_django/file/upload.html", context
            )


admin.site.register(Meta, MetaAdmin)
admin.site.register(File, FileAdmin)
admin.site.register(Stream, StreamAdmin)
admin.site.register(Video, VideoAdmin)
admin.site.register(Audio, AudioAdmin)
admin.site.register(Subtitle, SubtitleAdmin)
admin.site.register(Container, ContainerAdmin)



from django.db import models

from .managers import StreamManager


class Meta(models.Model):
    id = models.AutoField(primary_key=True)
    version = models.CharField(max_length=64)
    root = models.CharField(max_length=1024)

    class Meta:
        db_table = "metas"
        managed = False


class File(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=512)
    path = models.CharField(max_length=512)
    mtime = models.DateTimeField()

    class Meta:
        db_table = "files"
        managed = False


class Info(models.Model):
    id = models.AutoField(primary_key=True)
    file = models.ForeignKey(File, on_delete=models.CASCADE)
    duration = models.IntegerField()
    seekable = models.BooleanField()
    live = models.BooleanField()
    audio_streams = models.IntegerField()
    video_streams = models.IntegerField()
    subtitle_streams = models.IntegerField()

    class Meta:
        db_table = "infos"
        managed = False


class Stream(models.Model):
    VIDEO = "video"
    AUDIO = "audio"
    SUBTITLE = "subtitle"
    CONTAINER = "container"
    STREAM_TYPE_CHOICES = (
        (VIDEO, "Video"),
        (AUDIO, "Audio"),
        (SUBTITLE, "Subtitle"),
        (CONTAINER, "Container"),
    )

    id = models.AutoField(primary_key=True)
    info = models.ForeignKey(Info, on_delete=models.CASCADE)
    parent = models.ForeignKey(
        "self", related_name="children", on_delete=models.CASCADE
    )
    media_type = models.CharField(max_length=128)
    type = models.CharField(max_length=32, choices=STREAM_TYPE_CHOICES)
    # Common fields for a single table inheritance
    bitrate = models.IntegerField()
    max_bitrate = models.IntegerField()
    depth = models.IntegerField()
    language = models.CharField(max_length=256)
    # Video fields
    framerate_denom = models.IntegerField()
    framerate_num = models.IntegerField()
    height = models.IntegerField()
    par_denom = models.IntegerField()
    par_num = models.IntegerField()
    width = models.IntegerField()
    is_image = models.BooleanField()
    is_interlaced = models.BooleanField()
    # Audio fields
    channel_mask = models.IntegerField()
    channels = models.IntegerField()
    sample_rate = models.IntegerField()
    # Subtitle fields
    # Container fields
    class Meta:
        db_table = "streams"
        managed = False


class Video(Stream):
    objects = StreamManager()

    class Meta:
        proxy = True


class Audio(Stream):
    objects = StreamManager()

    class Meta:
        proxy = True


class Subtitle(Stream):
    objects = StreamManager()

    class Meta:
        proxy = True


class Container(Stream):
    objects = StreamManager()

    class Meta:
        proxy = True


class Field(models.Model):
    id = models.AutoField(primary_key=True)
    stream = models.ForeignKey(Stream, on_delete=models.CASCADE)
    name = models.CharField(max_length=128)
    value = models.CharField(max_length=1024)

    class Meta:
        db_table = "fields"
        managed = False



class Router(object):
    def db_for_read(self, model, **hints):
        if model._meta.app_label == "flumes_django":
            return "flumes"
        return None

    def db_for_write(self, model, **hints):
        # We don't allow to write, maybe later, but not now.
        return None

    def allow_relation(self, obj1, obj2, **hints):
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if app_label == "flumes_django":
            return False
        return None



from flumes.config import Config


class FlumesDjangoConfig(Config):
    def get_django_database_engine(self):
        driver = self.get_database_driver()
        if "postgresql" in driver:
            return "django.db.backends.postgresql_psycopg2"
        elif "mysql" in driver:
            return "django.db.backends.mysql"
        elif "sqlite" in driver:
            return "django.db.backends.sqlite3"
        else:
            return None



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class FlumesDjangoAppConfig(AppConfig):
    name = "flumes_django"

    def ready(self):
        # Checking settings
        if "flumes" not in settings.DATABASES:
            raise ImproperlyConfigured("Missing 'flumes' database")
        if "flumes_django.router.Router" not in settings.DATABASE_ROUTERS:
            raise ImproperlyConfigured("Missing 'flumes_django.router.Router' router")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flumes_django', 'flumes_django.templatetags', 'flumes_django.tests']

package_data = \
{'': ['*'],
 'flumes_django': ['static/admin/flume_django/common/*',
                   'templates/admin/flumes_django/file/*',
                   'templates/admin/flumes_django/info/*',
                   'templates/admin/flumes_django/stream/*',
                   'templates/flumes_django/templatetags/*']}

install_requires = \
['Django>=2.2,<4.0', 'flumes>=0.1.3,<0.2.0']

setup_kwargs = {
    'name': 'flumes-django',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'Jorge Zapata',
    'author_email': 'jorgeluis.zapata@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from django import template

register = template.Library()


@register.inclusion_tag("flume_django/templatetags/children.html")
def children_tag(stream):
    children = stream.children.all()
    return {"children": children}
